<?php

namespace Drupal\yamaps\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'yamaps' formatter.
 *
 * @FieldFormatter(
 *   id = "yamaps_default",
 *   label = @Translation("Yandex Map Field default"),
 *   field_types = {
 *     "yamaps"
 *   }
 * )
 */

class YamapsDefaultFormatter extends FormatterBase {
  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($items as $delta => $item) {

      $map_id = $items->getName() . '-' . $delta;
      // Map information.
      $default_js = yamaps_format_values_to_js($item);

      // Map initialization parameters.
      $map = [
        'init' => [
          'center' => $default_js['coords']['center'] ?? NULL,
          'zoom' => $default_js['coords']['zoom'] ?? NULL,
          'type' => 'yandex#map',
          'behaviors' => ['scrollZoom', 'dblClickZoom', 'drag'],
        ],
        'display_options' => [
          'display_type' => 'map',
          'width' => $this->getSetting('width'),
          'height' => $this->getSetting('height')
        ],
        'controls' => 1,
        'placemarks' => $default_js['placemarks'],
        'edit' => FALSE,
      ];
      $element = [
        '#theme' => 'yamaps_field_formatter',
        '#width' => $this->getSetting('width'),
        '#height' => $this->getSetting('height'),
        '#map_id' => $map_id,
      ];
      $elements['#attached']['drupalSettings']['yamaps'][$map_id] = $map;
      $elements[$delta] = $element;
    }

    $elements['#attached']['library'][] = 'yamaps/yandex-map-api';
    $elements['#attached']['library'][] = 'yamaps/yamaps-placemark';
    $elements['#attached']['library'][] = 'yamaps/yamaps-map';

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['width'] = [
      '#title' => $this->t('Map width'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('width'),
    ];

    $element['height'] = [
      '#title' => $this->t('Map height'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('height'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'width' => '100%',
        'height' => '400px',
      ] + parent::defaultSettings();
  }
}
