<?php

namespace Drupal\yamaps\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'yamaps_default' widget.
 *
 * @FieldWidget(
 *   id = "yamaps_default",
 *   label = @Translation("Yandex Map Field default"),
 *   field_types = {
 *     "yamaps"
 *   }
 * )
 */
class YamapsDefaultWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $map_id = $items->getName() . '-' . $delta;
    $element += [
      '#type' => 'fieldset',
      '#title' => $this->t('Map'),
    ];

    $value = $items[$delta] ?? [];

    // Map information.
    $default_js = yamaps_format_values_to_js($value);

    $element['map_container'] = [
      '#title' => $this->t('Preview'),
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#attributes' => [
        'id' => $map_id,
        'class' => 'yamaps-field-map',
        'style' => 'width: 100%; height: 400px;',
      ]
    ];


    $element['coords'] = [
      '#type' => 'textfield',
      '#title' => t('Coordinates'),
      '#default_value' => $items[$delta]->coords,
      '#attributes' => [
        'class' => ['field-yamaps-coords-' . $map_id],
      ],
    ];

    $element['placemarks'] = [
      '#type' => 'hidden',
      '#default_value' => $items[$delta]->placemarks,
      '#attributes' => ['class' => ['field-yamaps-placemarks-' . $map_id]],
    ];

    $element['type'] = [
      '#type' => 'hidden',
      '#default_value' => 'yandex#map',
    ];


    // Map initialization parameters.
    $map = [
      'init' => [
        'center' => $default_js['coords']['center'] ?? null,
        'zoom' => $default_js['coords']['zoom'] ?? null,
        'type' => 'yandex#map',
        'behaviors' => ['scrollZoom', 'dblClickZoom', 'drag'],
      ],
      'display_options' => [
        'display_type' => 'map',
        'open_button_text' => 'Open map',
        'close_button_text' => 'Close map',
        'width' => '100%',
        'height' => '400px'
      ],
      'controls' => 1,
      'placemarks' => $default_js['placemarks'],
      'edit' => TRUE,
    ];

    $element['#attached']['library'][] = 'yamaps/yandex-map-api';
    $element['#attached']['library'][] = 'yamaps/yamaps-placemark';
    $element['#attached']['library'][] = 'yamaps/yamaps-map';
    $element['#attached']['drupalSettings']['yamaps'] = [$map_id => $map];

    return $element;
  }
}
